@@ -1,9 +1,8 @@
-# Copyright 2021-2022 Teemu Ikonen
+# Copyright 2021-2023 Teemu Ikonen
 # SPDX-License-Identifier: GPL-3.0-only
 
 import argparse
-import gi
-import gpxpy
+import importlib.resources as resources
 import os
 import re
 import signal
@@ -12,26 +11,28 @@ import time
 import tokenize
 from datetime import datetime
 
-import importlib.resources as resources
+import gi
+import gpxpy
 
 import satellite.nmea as nmea
 import satellite.quectel as quectel
+from satellite import __version__
+
+from .mm_glib_source import ModemManagerGLibNmeaSource
 from .nmeasource import (
-    ModemNoNMEAError,
-    ModemLockedError,
-    ModemError,
-    NmeaSourceNotFoundError,
-    QuectelNmeaSource,
     GnssShareNmeaSource,
+    ModemError,
+    ModemLockedError,
+    ModemNoNMEAError,
+    NmeaSourceNotFoundError,
 )
 from .util import bearing_to_arrow, have_touchscreen, now, unique_filename
-from .widgets import text_barchart, DataFrame
-from satellite import __version__
+from .widgets import DataFrame, text_barchart
 
 gi.require_version('Gtk', '3.0')
 gi.require_version('Gdk', '3.0')
 gi.require_version('Handy', '1')
-from gi.repository import Gdk, Gio, GLib, Gtk, Handy  # noqa: E402
+from gi.repository import GLib, Gdk, Gio, Gtk, Handy  # noqa: E402, I100
 
 appname = 'Satellite'
 app_id = 'page.codeberg.tpikonen.satellite'
@@ -46,17 +47,21 @@ class SatelliteApp(Gtk.Application):
         Handy.init()
 
         desc = "Displays navigation satellite data and saves GPX tracks"
-        parser = argparse.ArgumentParser(description=desc)
+        parser = argparse.ArgumentParser(
+            description=desc, formatter_class=argparse.RawTextHelpFormatter)
         parser.add_argument(
             '-c', '--console-output', dest='console_output',
             action='store_true', default=False,
             help='Output satellite data to console')
         parser.add_argument(
             '-s', '--source', dest='source',
-            default='quectel',
-            help='Select NMEA source. Options are '
-            '\'quectel\' (default) for Quectel Modems or '
-            '\'gnss-share\' when using gnss-share')
+            choices=['auto', 'quectel', 'mm', 'gnss-share'],
+            default='auto',
+            help="Select NMEA source. Options are:\n"
+                 "'auto' (default) Automatic source detection\n"
+                 "'quectel' ModemManager with Quectel quirks\n"
+                 "'mm' ModemManager without quirks\n"
+                 "'gnss-share' Read from gnss-share socket\n")
         self.args = parser.parse_args()
 
         GLib.unix_signal_add(GLib.PRIORITY_DEFAULT, signal.SIGINT,
@@ -90,7 +95,7 @@ class SatelliteApp(Gtk.Application):
 
         self.source = None
 
-        self.infolabel.set_markup("<tt>" + "\n"*10 + "</tt>")
+        self.infolabel.set_markup("<tt>" + "\n" * 10 + "</tt>")
 
         self.dataframe = DataFrame()
 #        self.dataframe.header.set_text("Satellite info")
@@ -117,7 +122,7 @@ class SatelliteApp(Gtk.Application):
         self.last_data = None
         self.last_speed = None
         self.last_update = None
-        self.source_lost = False
+        self.had_error = False
         self.sigint_received = False
         self.refresh_rate = 1  # Really delay between updates in seconds
 
@@ -161,8 +166,6 @@ class SatelliteApp(Gtk.Application):
 
     def on_startup(self, app):
         self.create_actions()
-        # Initialize modem after GUI startup
-        GLib.idle_add(self.init_source)
 
     def on_activate(self, app):
         self.setup_styles()
@@ -171,33 +174,56 @@ class SatelliteApp(Gtk.Application):
         if have_touchscreen():
             self.datascroll.connect('edge-overshot', self.on_edge_overshot)
 
+        self.log_msg(f"{appname} version {__version__} started")
+        # Initialize modem after GUI startup
+        GLib.timeout_add(1000, self.init_source, None)
+
     def on_shutdown(self, app):
-        """Called after main loop exits."""
         print("Cleaning up...")
         self.gpx_write()
         if self.source is not None:
             self.source.close()
         print("...done.")
 
-    def init_source(self):
-        self.log_msg(f"{appname} version {__version__} started")
+    def init_source(self, unused):
         source_init = False
 
-        self.log_msg(f'Trying to initialize source "{self.args.source}"')
-
-        if self.args.source == 'quectel':
-            source_init = self.init_quectel_source()
-        elif self.args.source == 'gnss-share':
-            source_init = self.init_gnss_share_source()
-
-        if not source_init:
-            self.log_msg('No NmeaSource initialized')
-            return False  # Remove from idle_add
+        if self.args.source == 'auto':
+            self.log_msg("Detecting NMEA sources...")
+            if not source_init:
+                source_init = self.init_gnss_share_source(autodetect=True)
+            if not source_init:
+                source_init = self.init_mm_source(
+                    quirks=['detect'], autodetect=True)
+            if not source_init:
+                self.log_msg('NMEA source not found')
+                dialog = Gtk.MessageDialog(
+                    parent=self.window, modal=True,
+                    message_type=Gtk.MessageType.ERROR,
+                    buttons=Gtk.ButtonsType.OK,
+                    text="Could not find an NMEA source")
+                dialog.set_title("Error initializing NMEA source")
+                dialog.run()
+                dialog.destroy()
+                return GLib.SOURCE_REMOVE
+        else:
+            self.log_msg(f'NMEA source "{self.args.source}" selected')
+            if self.args.source == 'quectel':
+                source_init = self.init_mm_source(quirks=['QuectelTalker'])
+            elif self.args.source == 'mm':
+                source_init = self.init_mm_source()
+            elif self.args.source == 'gnss-share':
+                source_init = self.init_gnss_share_source()
+            if not source_init:
+                self.log_msg('Could not initialize NMEA source')
+                return GLib.SOURCE_REMOVE
 
         self.log_msg(
-            f"Source is {self.source.manufacturer}, model {self.source.model}"
-            + f", revision {self.source.revision}"
-            if self.source.revision else "")
+            f"Source is {self.source.manufacturer}"
+            + (f", model {self.source.model}" if self.source.model else "")
+            + (f", revision {self.source.revision}" if self.source.revision else "")
+            + (f" using {', '.join(self.source.quirks)} quirks"
+               if hasattr(self.source, "quirks") and self.source.quirks else ""))
 
         if (self.source.model and self.source.model.startswith("QUECTEL")):
             constellations = quectel.get_constellations(self.source)
@@ -218,14 +244,15 @@ class SatelliteApp(Gtk.Application):
 
         return GLib.SOURCE_REMOVE
 
-    def init_gnss_share_source(self):
+    def init_gnss_share_source(self, autodetect=False):
         try:
             self.source = GnssShareNmeaSource(self.location_update_cb)
             self.source.initialize()
         except Exception as e:
-            fatal = False
+            if autodetect:
+                return False
             self.log_msg(str(e))
-            dtext = 'Can you access "/var/run/gnss-share.sock"?\n' + str(e)
+            dtext = str(e)
             dialog = Gtk.MessageDialog(
                 parent=self.window, modal=True,
                 message_type=Gtk.MessageType.ERROR,
@@ -233,44 +260,40 @@ class SatelliteApp(Gtk.Application):
             dialog.set_title("Error initializing NMEA source")
             dialog.run()
             dialog.destroy()
-            if fatal:
-                self.quit()
 
             return False
 
         return True
 
-    def init_quectel_source(self):
+    def init_mm_source(self, quirks=[], autodetect=False):
         try:
-            self.source = QuectelNmeaSource(
+            self.source = ModemManagerGLibNmeaSource(
                 self.location_update_cb,
                 refresh_rate=self.refresh_rate,
+                quirks=quirks,
                 # save_filename=unique_filename(self.gpx_save_dir + '/nmeas',
                 #                               '.txt')
             )
             self.source.initialize()
         except Exception as e:
-            fatal = False
+            if autodetect:
+                return False
             if isinstance(e, ModemLockedError):
                 self.log_msg("Modem is locked")
                 dtext = "Please unlock the Modem"
             else:
-                fatal = isinstance(e, gi.repository.GLib.GError)
-                self.log_msg("Error initializing NMEA source")
                 etext = str(e)
+                self.log_msg(f"Error initializing ModemManager NMEA source: {etext}")
                 dtext = etext if etext else (
-                    "Could not find or initialize NMEA source")
+                    "Could not initialize ModemManager NMEA source")
             dialog = Gtk.MessageDialog(
                 parent=self.window, modal=True,
                 message_type=Gtk.MessageType.ERROR,
-                buttons=Gtk.ButtonsType.CLOSE if fatal else Gtk.ButtonsType.OK,
+                buttons=Gtk.ButtonsType.OK,
                 text=dtext)
             dialog.set_title("Error initializing NMEA source")
             dialog.run()
             dialog.destroy()
-            if fatal:
-                self.quit()
-                return
 
             return False
 
@@ -301,7 +324,7 @@ class SatelliteApp(Gtk.Application):
             version=__version__,
             comments="A program for showing navigation satellite data",
             license_type=Gtk.License.GPL_3_0_ONLY,
-            copyright="Copyright 2021-2022 Teemu Ikonen",
+            copyright="Copyright 2021-2023 Teemu Ikonen",
         )
         adlg.present()
 
@@ -407,6 +430,12 @@ class SatelliteApp(Gtk.Application):
             fixage = to_str(data.get("fixage"), "%0.0f s")
             return "%s / %s" % (up_age, fixage)
 
+        def get_dops(xkey):
+            pdop = to_str(data.get("pdop"), "%1.1f")
+            hdop = to_str(data.get("hdop"), "%1.1f")
+            vdop = to_str(data.get("vdop"), "%1.1f")
+            return f"{pdop} / {hdop} / {vdop}"
+
         mode2fix = {
             "2": "2 D",
             "3": "3 D",
@@ -415,19 +444,19 @@ class SatelliteApp(Gtk.Application):
         # Mapping: Data key, description, converter func
         order = [
             ("mode", "Fix type", lambda x: mode2fix.get(x, "No Fix")),
-            ("mode_indicator", "Modes (GP,GL,GA)", lambda x: str(x)),
+            ("mode_indicator", "Modes (GP,GL,GA)",
+                lambda x: str(x) if x is not None else "n/a"),
             ("actives", "Active / in use sats", get_actives),
             ("visibles", "Receiving sats", lambda x: str(len(
-                list(r for r in x if r['snr'] > 0.0)))),
+                [r for r in x if r['snr'] > 0.0]))),
             ("visibles", "Visible sats", lambda x: str(len(x))),
             # ("fixage", "Age of fix", lambda x: to_str(x, "%0.0f s")),
             ("fixage", "Age of update / fix", get_ages),
             ("systime", "Sys. Time", lambda x: x.strftime(utcfmt)),
-            ("latlon",  "Latitude",
-                lambda x: "%0.6f" % x[0] if x else "-"),
-            ("latlon",  "Longitude",
-                lambda x: "%0.6f" % x[1] if x else "-"),
-            ("altitude",  "Altitude", lambda x: to_str(x, "%0.1f m")),
+            ("latlon", "Latitude", lambda x: "%0.6f" % x[0] if x else "-"),
+            ("latlon", "Longitude", lambda x: "%0.6f" % x[1] if x else "-"),
+            ("altitude", "Altitude", lambda x: to_str(x, "%0.1f m")),
+            ("geoid_sep", "Geoidal separation", lambda x: to_str(x, "%0.1f m")),
             # ("fixtime", "Time of fix",
             #     lambda x: x.strftime(utcfmt) if x else "-"),
             # ("date",    "Date of fix",
@@ -436,9 +465,7 @@ class SatelliteApp(Gtk.Application):
             ("true_course", "True Course",
                 lambda x: to_str(x, "%0.1f deg ")
                 + (bearing_to_arrow(x) if x is not None else "")),
-            ("pdop", "PDOP", lambda x: to_str(x)),
-            ("hdop", "HDOP", lambda x: to_str(x)),
-            ("vdop", "VDOP", lambda x: to_str(x)),
+            ("pdop", "PDOP/HDOP/VDOP", get_dops),
         ]
         descs = []
         vals = []
@@ -471,10 +498,9 @@ class SatelliteApp(Gtk.Application):
         self.last_mode = mode
 
     def set_speedlabel(self, speed, bearing=None):
-        spd = str(int(3.6*speed)) if speed else "-"
+        spd = str(int(3.6 * speed)) if speed else "-"
         arrow = bearing_to_arrow(bearing) if bearing is not None else ""
-        speedfmt = ('<span size="50000">%s%s</span>\n' +
-                    '<span size="30000">%s</span>')
+        speedfmt = '<span size="50000">%s%s</span>\n<span size="30000">%s</span>'
         speedstr = speedfmt % (spd, arrow, "km/h")
         self.speedlabel.set_markup(speedstr)
 
@@ -539,8 +565,14 @@ class SatelliteApp(Gtk.Application):
     def update(self):
         try:
             nmeas = self.source.get()
+            if self.had_error:
+                self.log_msg("Getting updates")
+                self.main_box.set_sensitive(True)
+
+            self.had_error = False
+            data = nmea.parse(nmeas)
         except Exception as e:
-            fatal = False
+            nmeas = None
             show_dialog = False
             etext = str(e)
             dtext = None
@@ -552,37 +584,29 @@ class SatelliteApp(Gtk.Application):
             elif isinstance(e, ModemError):
                 dtext = "Modem error: " + str(e)
             elif isinstance(e, NmeaSourceNotFoundError):
-                if not self.source_lost:
+                if not self.had_error:
                     dtext = etext if etext else "Modem disappeared"
-                    self.source_lost = True
+                    self.had_error = True
                     self.main_box.set_sensitive(False)
             else:
                 dtext = etext if etext else "Unknown error"
-            if show_dialog:
-                dialog = Gtk.MessageDialog(
-                    parent=self.window, modal=True,
-                    message_type=Gtk.MessageType.ERROR,
-                    buttons=Gtk.ButtonsType.OK, text=dtext)
-                dialog.set_title("Unrecoverable error" if fatal else "Error")
-                dialog.run()
-                dialog.destroy()
-                if fatal:
-                    self.quit()
-                    return
-            elif dtext is not None:
-                self.log_msg(dtext)
-            return
+            if not self.had_error:
+                if show_dialog:
+                    dialog = Gtk.MessageDialog(
+                        parent=self.window, modal=True,
+                        message_type=Gtk.MessageType.ERROR,
+                        buttons=Gtk.ButtonsType.OK, text=dtext)
+                    dialog.set_title("Error")
+                    dialog.run()
+                    dialog.destroy()
+                elif dtext is not None:
+                    self.log_msg(dtext)
+            self.had_error = True
+            if self.last_data is None:
+                return
+            else:
+                data = self.last_data
 
-        if not nmeas:
-            return
-
-        if self.source_lost:
-            self.log_msg("Modem appeared")
-            self.main_box.set_sensitive(True)
-
-        self.source_lost = False
-
-        data = nmea.parse(nmeas)
         data["updateage"] = ((time.time() - self.last_update)
                              if self.last_update else None)
 
@@ -603,7 +627,7 @@ class SatelliteApp(Gtk.Application):
 
         # log
         mode = data["mode"]
-        mode = int(mode) if mode else 0
+        mode = int(mode) if mode else self.last_mode
         if mode != self.last_mode:
             if mode > 1:
                 self.log_msg(f"Got lock, mode: {mode}")
