@@ -1,10 +1,11 @@
-# Copyright 2021-2022 Teemu Ikonen
+# Copyright 2021-2023 Teemu Ikonen
 # SPDX-License-Identifier: GPL-3.0-only
 
 import datetime
-import pynmea2
 import re
 
+import pynmea2
+
 MS_PER_KNOT = 0.514444
 
 lastfix_dt = None
@@ -28,7 +29,7 @@ def fget(key, scale=1.0):
 def iget(key, default=None):
     def fn(d):
         try:
-            return int(d.get(key))
+            return int(d.get(key, default))
         except ValueError:
             return default
     return fn
@@ -61,9 +62,9 @@ def get_latlon(mdict):
     lat_min = float(lat[2:])
     lon_deg = float(lon[:3])
     lon_min = float(lon[3:])
-    flat = lat_deg + lat_min/60
+    flat = lat_deg + lat_min / 60
     flat = -1 * flat if lat_dir == 'S' else flat
-    flon = lon_deg + lon_min/60
+    flon = lon_deg + lon_min / 60
     flon = -1 * flon if lon_dir == 'W' else flon
     return (flat, flon)
 
@@ -98,12 +99,10 @@ getters = {
         'GGA': get_altitude_gga,
         'GNS': fget('altitude'),
     },
-    "fixtime": {
+    "fixtime": {  # Time of position report
         'RMC': get_time,
-        'GGA': get_time,
-    },
-    "time": {  # Reported also when no fix
         'GNS': get_time,
+        'GGA': get_time,
     },
     "date": {
         'RMC': get_date,
@@ -131,6 +130,7 @@ getters = {
     },
     "num_sats": {
         'GNS': iget('num_sats', default=0),
+        'GGA': iget('num_sats', default=0),
     },
     "pdop": {
         'GSA': fget('pdop'),
@@ -143,7 +143,8 @@ getters = {
     "vdop": {
         'GSA': fget('vdop'),
     },
-    "geo_sep": {
+    "geoid_sep": {
+        'GGA': fget('geo_sep'),
         'GNS': fget('geo_sep'),
     },
     "sel_mode": {
@@ -182,7 +183,7 @@ def parse(nmeas, always_add_prefix=False):
         return float(s) if s else empty_val
 
     def add_prn_prefix(prns, talker, always=always_add_prefix):
-        """Add constellation prefix to PRN string"""
+        """Add constellation prefix to PRN string."""
         beidou_prefix = "C"
         galileo_prefix = "E"
         glonass_prefix = "R"
@@ -227,7 +228,7 @@ def parse(nmeas, always_add_prefix=False):
                         'snr': fl(getattr(msg, f'snr_{n}', None), 0.0),
                     })
         elif isinstance(msg, pynmea2.types.GSA):
-            for n in range(1, 12+1):
+            for n in range(1, 12 + 1):
                 prns = getattr(msg, f'sv_id{n:02d}')
                 if prns and prns.isdigit():
                     actives.append(add_prn_prefix(prns, msg.talker))
@@ -248,13 +249,13 @@ def parse(nmeas, always_add_prefix=False):
     }
     out.update({k: msg_get(msgs, k) for k in getters.keys()})
 
-    datenow = datetime.datetime.utcnow()
+    datenow = datetime.datetime.now(datetime.timezone.utc)
     fixtime = out.get('fixtime')
     fixdate = out.get('date')
     if fixdate is None and fixtime is not None:
         # We have a fix but no RMC sentence
         fixdate = datenow.date()
-    fixdt = (datetime.datetime.combine(fixdate, fixtime)
+    fixdt = (datetime.datetime.combine(fixdate, fixtime, datetime.timezone.utc)
              if (fixtime and fixdate) else None)
     out["datetime"] = fixdt
     out["systime"] = datenow
