@@ -1,11 +1,10 @@
-# Copyright 2021-2022 Teemu Ikonen
+# Copyright 2021-2023 Teemu Ikonen
 # SPDX-License-Identifier: GPL-3.0-only
 
-import gi
 import os
-
 from datetime import datetime, timezone
 
+import gi
 gi.require_version('Gdk', '3.0')
 from gi.repository import Gdk  # noqa: E402
 
@@ -16,15 +15,13 @@ week_now = int((now.timestamp() - gps_epoch.timestamp()) / one_week)
 
 
 def have_touchscreen():
-    """Return True if the default seat of default display has touch capability
-    """
+    """Return True if the default seat of default display has touch capability."""
     return bool(Gdk.Display.get_default_seat(
         Gdk.Display.get_default()).get_capabilities() & Gdk.SeatCapabilities.TOUCH)
 
 
 def datetime_from_gpstime(week, millisecs, fix_week=False):
-    """Return a datetime object formed from GPS week number and
-    milliseconds from week start.
+    """Return a datetime from GPS week number and milliseconds from week start.
 
     If fix_week is True, set the bits above 10 in week number from
     current date, see
@@ -38,7 +35,7 @@ def datetime_from_gpstime(week, millisecs, fix_week=False):
 
 
 def gpstime_from_datetime(dt):
-    """Return a (gps_week, millisec) tuple from a datetime object"""
+    """Return a (gps_week, millisec) tuple from a datetime object."""
     if dt < gps_epoch:
         raise ValueError("Time cannot be less than GPS epoch")
     ts = dt.timestamp()
@@ -51,7 +48,7 @@ def gpstime_from_datetime(dt):
 def unique_filename(namestem, ext, timestamp=False):
     if timestamp:
         namestem += "-" + datetime.now().isoformat(
-                          '_', 'seconds').replace(':', '.')
+            '_', 'seconds').replace(':', '.')
     name = None
     for count in ('~%d' % n if n > 0 else '' for n in range(100)):
         test = namestem + count + ext
@@ -77,7 +74,7 @@ def bearing_to_arrow(bearing):
         '\u2196',
         '\u2191',
     ]
-    edges = list(22.5 + 45.0 * n for n in range(0, 8)) + [360.0]
+    edges = [22.5 + 45.0 * n for n in range(0, 8)] + [360.0]
 
     angle = bearing - (bearing // 360) * 360
     index = next(ind for (ind, e) in enumerate(edges) if angle < e)
