@@ -1,10 +1,9 @@
-# Copyright 2021-2022 Teemu Ikonen
+# Copyright 2021-2023 Teemu Ikonen
 # SPDX-License-Identifier: GPL-3.0-only
 
-import gi
-
 import importlib.resources as resources
 
+import gi
 gi.require_version('Gtk', '3.0')
 gi.require_version('Gdk', '3.0')
 from gi.repository import Gtk  # noqa: E402
@@ -18,8 +17,7 @@ def text_barchart(data, highlights, height=None, width=30):
     height  Number of lines in the generated bar chart
     width   Width of the generated bar chart in chars
     """
-    sdata = list((d[0] if d[0] else '',
-                 int(d[1]) if d[1] else 0) for d in data)
+    sdata = [(d[0] if d[0] else '', int(d[1]) if d[1] else 0) for d in data]
     sdata.sort(key=lambda x: x[1], reverse=True)
 
     dstr = ''
@@ -41,14 +39,14 @@ def text_barchart(data, highlights, height=None, width=30):
     cmax_xaxis = cmaxbar + 3
     for d in sdata[:barlines]:
         block = '\u2585' if d[0] in highlights else '='
-        dstr += "%3s\u2502%s %d\n" % (d[0], block*int(scale*d[1]), d[1])
+        dstr += "%3s\u2502%s %d\n" % (d[0], block * int(scale * d[1]), d[1])
     if barlines < len(sdata):
         dstr += "   \u256a\n"
     elif (len(sdata) - axislines) < height:
         # Add empty lines to y-axis
         dstr += '   \u2502\n' * (height - len(sdata) - axislines)
-    dstr += "   \u251c" + '\u2500'*(cmax_xaxis) + '\u2524\n'
-    dstr += "   0" + ' '*(cmax_xaxis - 1) + str(max_xaxis)
+    dstr += "   \u251c" + '\u2500' * (cmax_xaxis) + '\u2524\n'
+    dstr += "   0" + ' ' * (cmax_xaxis - 1) + str(max_xaxis)
     return dstr
 
 
